@@ -15,7 +15,7 @@ module MCollective
     attr_reader :main_collective, :ssl_cipher, :registration_collective
     attr_reader :direct_addressing, :direct_addressing_threshold, :ttl
     attr_reader :default_discovery_method, :default_discovery_options
-    attr_reader :publish_timeout, :threaded
+    attr_reader :publish_timeout, :threaded, :soft_shutdown
 
     def initialize
       @configured = false
@@ -35,13 +35,14 @@ module MCollective
               key = $1.strip
               val = $2
 
-              case key
+              begin
+                case key
                 when "registration"
                   @registration = val.capitalize
                 when "registration_collective"
                   @registration_collective = val
                 when "registerinterval"
-                  @registerinterval = val.to_i
+                  @registerinterval = Integer(val)
                 when "collectives"
                   @collectives = val.split(",").map {|c| c.strip}
                 when "main_collective"
@@ -49,9 +50,9 @@ module MCollective
                 when "logfile"
                   @logfile = val
                 when "keeplogs"
-                  @keeplogs = val.to_i
+                  @keeplogs = Integer(val)
                 when "max_log_size"
-                  @max_log_size = val.to_i
+                  @max_log_size = Integer(val)
                 when "loglevel"
                   @loglevel = val
                 when "logfacility"
@@ -71,7 +72,7 @@ module MCollective
                 when "direct_addressing"
                   @direct_addressing = Util.str_to_bool(val)
                 when "direct_addressing_threshold"
-                  @direct_addressing_threshold = val.to_i
+                  @direct_addressing_threshold = Integer(val)
                 when "color"
                   @color = Util.str_to_bool(val)
                 when "daemonize"
@@ -87,7 +88,7 @@ module MCollective
                 when /^plugin.(.+)$/
                   @pluginconf[$1] = val
                 when "publish_timeout"
-                  @publish_timeout = val.to_i
+                  @publish_timeout = Integer(val)
                 when "rpcaudit"
                   @rpcaudit = Util.str_to_bool(val)
                 when "rpcauditprovider"
@@ -101,19 +102,26 @@ module MCollective
                 when "logger_type"
                   @logger_type = val
                 when "fact_cache_time"
-                  @fact_cache_time = val.to_i
+                  @fact_cache_time = Integer(val)
                 when "ssl_cipher"
                   @ssl_cipher = val
                 when "threaded"
                   @threaded = Util.str_to_bool(val)
                 when "ttl"
-                  @ttl = val.to_i
+                  @ttl = Integer(val)
                 when "default_discovery_options"
                   @default_discovery_options << val
                 when "default_discovery_method"
                   @default_discovery_method = val
+                when "soft_shutdown"
+                  @soft_shutdown = Util.str_to_bool(val)
+                when "topicprefix", "topicsep", "queueprefix", "rpchelptemplate", "helptemplatedir"
+                  Log.warn("Use of deprecated '#{key}' option.  This option is ignored and should be removed from '#{configfile}'")
                 else
                   raise("Unknown config parameter '#{key}'")
+                end
+              rescue ArgumentError => e
+                raise "Could not parse value for configuration option '#{key}' with value '#{val}'"
               end
             end
           end
@@ -121,9 +129,6 @@ module MCollective
 
         raise('The %s config file does not specify a libdir setting, cannot continue' % configfile) if @libdir.empty?
 
-        I18n.load_path = Dir[File.expand_path(File.join(File.dirname(__FILE__), "locales", "*.yml"))]
-        I18n.locale = :en
-
         read_plugin_config_dir("#{@configdir}/plugin.d")
 
         raise 'Identities can only match /\w\.\-/' unless @identity.match(/^[\w\.\-]+$/)
@@ -143,7 +148,7 @@ module MCollective
         PluginManager.loadclass("Mcollective::Audit::#{@rpcauditprovider}") if @rpcaudit
         PluginManager << {:type => "global_stats", :class => RunnerStats.new}
 
-        Log.logmsg(:PLMC1, "The Marionette Collective version %{version} started by %{name} using config file %{config}", :info, :version => MCollective::VERSION, :name => $0, :config => configfile)
+        Log.info("The Marionette Collective version #{MCollective::VERSION} started by #{$0} using config file #{configfile}")
       else
         raise("Cannot find config file '#{configfile}'")
       end
@@ -187,6 +192,7 @@ module MCollective
       @mode = :client
       @publish_timeout = 2
       @threaded = false
+      @soft_shutdown = false
     end
 
     def read_plugin_config_dir(dir)
