@@ -24,17 +24,21 @@ module MCollective
         @results[idx] = item
       end
 
+      def fetch(key, default)
+        @results.fetch(key, default)
+      end
+
       def each
         @results.each_pair {|k,v| yield(k,v) }
       end
 
       def to_json(*a)
         {:agent => @agent,
-          :action => @action,
-          :sender => @results[:sender],
-          :statuscode => @results[:statuscode],
-          :statusmsg => @results[:statusmsg],
-          :data => @results[:data]}.to_json(*a)
+         :action => @action,
+         :sender => @results[:sender],
+         :statuscode => @results[:statuscode],
+         :statusmsg => @results[:statusmsg],
+         :data => @results[:data]}.to_json(*a)
       end
     end
   end
